/*
 * CG Madness - a Marble Madness clone
 * Copyright (C) 2007  Sven Reinck <sreinck@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "NetworkBall.hpp"

#include "text/text.hpp"

#include "ColorStack.hpp"

#include "graphics/camera.hpp"
#include "graphics/texture.hpp"
#include "graphics/objects.hpp"

#include <cmath>

#define DEAD_TIME 60

int NetworkBall::sTextureBallDead = 0;

void NetworkBall::init()
{
	sTextureBallDead = loadTexture("data/ball_dead.tga", false);
}

NetworkBall::NetworkBall() :
	mDeadCounter(0)
{
  // empty
}

NetworkBall::~NetworkBall()
{
  // empty
}

void NetworkBall::alive()
{
	mDeadCounter = 0;
}

void NetworkBall::update(float interval)
{
	Ball::update(interval);

	mDeadCounter++;
}

void NetworkBall::drawBall() const
{
	ColorStack::colorStack.setColor(Color4::white);
	mBallObject.drawDefault();

	float distance = (mPos - sgCamera).len();

	if (distance < 12.0f)
	{
		glPushMatrix();
		{
			Matrix m;

			getCameraRotationMatrix(m);

			mOrientation.conjugate().transform();
			glMultMatrixf((GLfloat*) m);

			glTranslatef(-widthStrokeText(mName) / 2.0f, 1.5f, 0.0f);

			float alpha = 1.0f;
			if (distance > 10.0f)
			{
				float t = (distance - 10.0f) / 2.0f;
				alpha = 2 * t * t * t - 3 * t * t + 1.0f;
			}

			ColorStack::colorStack.setColor(Color4(Color4::white, alpha));
			drawStrokeThinText(mName);
		}
		glPopMatrix();
	}
}

void NetworkBall::drawExplosion() const
{
	drawBall();
}

void NetworkBall::activateBallShader() const
{
	Ball::activateBallShader();

	{
#if 0
		glActiveTexture(GL_TEXTURE1);
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, sTextureBallDead);

		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE);

		glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_INTERPOLATE);
		{
			glTexEnvi(GL_TEXTURE_ENV, GL_SRC0_RGB, GL_TEXTURE);
			glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB, GL_SRC_COLOR);

			glTexEnvi(GL_TEXTURE_ENV, GL_SRC1_RGB, GL_PREVIOUS);
			glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND1_RGB, GL_SRC_COLOR);

			glTexEnvi(GL_TEXTURE_ENV, GL_SRC2_RGB, GL_PRIMARY_COLOR);
			glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND2_RGB, GL_SRC_ALPHA);
		}

		float alpha = 1.0f / (1.0f + exp(0.05f * (DEAD_TIME - mDeadCounter)));
#else
		float alpha = mIsBallInPieces ? mExplosion.alpha() : 1.0f;
		ColorStack::colorStack.pushColor(Color4(mColor, alpha));
		glEnable(GL_BLEND);
#endif
	}
}

void NetworkBall::deactivateBallShader() const
{
#if 0
	glActiveTexture(GL_TEXTURE1);
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glDisable(GL_TEXTURE_2D);
	glActiveTexture(GL_TEXTURE0);
#else
	glDisable(GL_BLEND);
	ColorStack::colorStack.popColor();
#endif
	Ball::deactivateBallShader();
}
