/*
 * CG Madness - a Marble Madness clone
 * Copyright (C) 2007  Sven Reinck <sreinck@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "Console.hpp"

#include "objects.hpp"
#include "text/text.hpp"

#include "macros.hpp"

#include GL_H

#define PADDING_FACTOR 1.8f

struct Line
{
	String string;
	Color3 color;

	Line(const String& string, const Color3& color) :
		string(string), color(color)
	{
		// empty
	}
};

Console::Console(unsigned int maxLines) :
	mMaxLines(maxLines)
{
	// empty
}

Console::~Console()
{
	// empty
}

void Console::addLine(const String& string, const Color3& color)
{
	Line line(string, color);

	mLines.push_front(line);

	while (mLines.size() > mMaxLines)
	{
		mLines.pop_back();
	}
}

void Console::update(float interval)
{
	// empty
}

void Console::drawHUD(float width, float height) const
{
	drawRectangle(width, height);
	drawContent(width, height);
}

void Console::drawContent(float width, float height) const
{
	float lineHight = height / mMaxLines;
	float scale = lineHight / PADDING_FACTOR;
	float padding = (lineHight - scale) / 2.0f;

	int idxLine = 0;

	FOREACH(mLines, iter)
	{
		const Line& line = *iter;

		glPushMatrix();
		{
			glTranslatef(padding, padding + idxLine * lineHight, 0.0f);
			glScalef(scale, scale, scale);

			glColor3fv(&line.color.r);
			drawStrokeThinText(line.string);
		}
		glPopMatrix();

		idxLine++;
	}
}
