/*
 * CG Madness - a Marble Madness clone
 * Copyright (C) 2007  Sven Reinck <sreinck@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "Client.hpp"

#define CLIENT_DEBUG 0

int client_callback(grapple_message* message, void* context)
{
	((Client*) context)->callback(message);

	return 0;
}

#if (CLIENT_DEBUG)
#define GRAPPLE_DEBUG(x) { int result = (x); if (result != GRAPPLE_OK) { fprintf(stderr,"Client: %s => %s\n", #x, grapple_error_text(grapple_client_error_get(mClient))); } }
#else
#define GRAPPLE_DEBUG(x) x
#endif

Client::Client(const char* progName, const char* progVersion, const char* address, const char* clientName):
	mProgName(progName),
	mProgVersion(progVersion),
	mAddress(address),
	mClientName(clientName)
{
	start();
}

Client::~Client()
{
	grapple_client_stop(mClient);
	grapple_client_destroy(mClient);
}

void Client::loopMessages(MessageDispatcher* dispatcher)
{
	while (true)
	{
		grapple_message* msg = mMessages.pop();

		processMessage(dispatcher, msg);
	}
}

void Client::processMessages(MessageDispatcher* dispatcher)
{
	while (!mMessages.isEmpty())
	{
		grapple_message* msg = mMessages.pop();

		processMessage(dispatcher, msg);
	}
}

void Client::send(const ByteArray& package, grapple_user user)
{
	grapple_client_send(mClient, user, 0, package.data(), package.size());
}

void Client::broadcast(const ByteArray& package)
{
	send(package, GRAPPLE_EVERYONE);
}

void Client::broadcastOthers(const ByteArray& package)
{
	send(package, GRAPPLE_EVERYONEELSE);
}

void Client::callback(grapple_message* message)
{
	mMessages.push(message);
}

#define PUSH_MASSAGE(x) case x: dispatcher->log(#x);

void Client::processMessage(MessageDispatcher* dispatcher, grapple_message* message)
{
	switch (message->type)
	{
	case GRAPPLE_MSG_USER_MSG:
	{
		ByteArray data(message->USER_MSG.length, reinterpret_cast<unsigned char*>(message->USER_MSG.data));
		Package package(message->USER_MSG.id, data);
		dispatcher->package(package);
		break;
	}
	case GRAPPLE_MSG_NEW_USER:
	case GRAPPLE_MSG_NEW_USER_ME:
		dispatcher->newUser(message->NEW_USER.id, message->NEW_USER.me, message->NEW_USER.name);
		break;
	case GRAPPLE_MSG_USER_NAME:
		dispatcher->userName(message->USER_NAME.id, message->USER_NAME.name);
		break;
	case GRAPPLE_MSG_USER_DISCONNECTED:
		dispatcher->userDisconnected(message->USER_DISCONNECTED.id);
		break;
	case GRAPPLE_MSG_SESSION_NAME:
		dispatcher->log(String("session name: ") + message->SESSION_NAME.name);
		break;
	case GRAPPLE_MSG_SERVER_DISCONNECTED:
		dispatcher->serverDisconnected();
		break;
	case GRAPPLE_MSG_CONNECTION_REFUSED:
		dispatcher->connectionRefused(message->CONNECTION_REFUSED.reason);
		break;

		PUSH_MASSAGE(GRAPPLE_MSG_PING);
		break;
		PUSH_MASSAGE(GRAPPLE_MSG_GROUP_CREATE);
		break;
		PUSH_MASSAGE(GRAPPLE_MSG_GROUP_ADD);
		break;
		PUSH_MASSAGE(GRAPPLE_MSG_GROUP_REMOVE);
		break;
		PUSH_MASSAGE(GRAPPLE_MSG_GROUP_DELETE);
		break;
		PUSH_MASSAGE(GRAPPLE_MSG_YOU_ARE_HOST);
		break;
		PUSH_MASSAGE(GRAPPLE_MSG_CONFIRM_RECEIVED);
		break;
		PUSH_MASSAGE(GRAPPLE_MSG_CONFIRM_TIMEOUT);
		break;
		PUSH_MASSAGE(GRAPPLE_MSG_GAME_DESCRIPTION);
		break;
	}

	grapple_message_dispose(message);
}

void Client::run()
{
	mClient = grapple_client_init(mProgName, mProgVersion);

	GRAPPLE_DEBUG(grapple_client_address_set(mClient, mAddress));
	GRAPPLE_DEBUG(grapple_client_port_set(mClient, 1234));
	GRAPPLE_DEBUG(grapple_client_protocol_set(mClient, GRAPPLE_PROTOCOL_TCP));

	if (mClientName)
	{
		GRAPPLE_DEBUG(grapple_client_name_set(mClient, mClientName));
	}

	GRAPPLE_DEBUG(grapple_client_start(mClient, 0));

	GRAPPLE_DEBUG(grapple_client_callback_setall(mClient, client_callback, this));
}
