/*
 * CG Madness - a Marble Madness clone
 * Copyright (C) 2007  Sven Reinck <sreinck@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef Client_hpp
#define Client_hpp

#include "Network.hpp"
#include "thread/Thread.hpp"

#include "MessageDispatcher.hpp"

#include "thread/BlockingQueue.hpp"

#include "utils/String.hpp"

class Client : public Network, public Thread
{
public:
  Client(const char* progName, const char* progVersion, const char* address, const char* clientName = NULL);
  virtual ~Client();

  void loopMessages(MessageDispatcher* dispatcher);
  void processMessages(MessageDispatcher* dispatcher);

  void send(const ByteArray& package, grapple_user user);
  void broadcast(const ByteArray& package);
  void broadcastOthers(const ByteArray& package);

private:
	grapple_client mClient;

	BlockingQueue<grapple_message*> mMessages;

	String mProgName;
	String mProgVersion;
	String mAddress;
	String mClientName;

	void callback(grapple_message* message);

	void processMessage(MessageDispatcher* dispatcher, grapple_message* message);

	void run();

	friend int client_callback(grapple_message* message, void* context);

};

#endif
