/*
 * CG Madness - a Marble Madness clone
 * Copyright (C) 2007  Sven Reinck <sreinck@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "Server.hpp"

#define SERVER_DEBUG 0

int server_callback(grapple_message* message, void* context)
{
	((Server*) context)->callback(message);

	return 0;
}

#if (SERVER_DEBUG)
#define GRAPPLE_DEBUG(x) { int result = (x); if (result != GRAPPLE_OK) { fprintf(stderr,"Server: %s => %s\n", #x, grapple_error_text(grapple_server_error_get(mServer))); } }
#else
#define GRAPPLE_DEBUG(x) x
#endif

Server::Server(const char* progName, const char* progVersion, MessageDispatcher* dispatcher, BooleanCondition* running) :
	mDispatcher(dispatcher), mRunning(running)
{
	mServer = grapple_server_init(progName, progVersion);

#if 0
	grapple_server_ip_set(mServer, "127.0.0.1");	/* optional */
#endif
	GRAPPLE_DEBUG(grapple_server_port_set(mServer, 1234));
	GRAPPLE_DEBUG(grapple_server_protocol_set(mServer, GRAPPLE_PROTOCOL_TCP));
	GRAPPLE_DEBUG(grapple_server_session_set(mServer, "session"));
	GRAPPLE_DEBUG(grapple_server_start(mServer));

	GRAPPLE_DEBUG(grapple_server_callback_setall(mServer, server_callback, this));

	mDispatcher->log("Server started ...");
	mRunning->setValue(true);
}

Server::~Server()
{
	grapple_server_stop(mServer);
	grapple_server_destroy(mServer);

	mDispatcher->log("Server stopped ...");
	mRunning->setValue(false);
}

void Server::loopMessages()
{
	while (true)
	{
		grapple_message* msg = mMessages.pop();

		processMessage(msg);
	}
}

void Server::send(const ByteArray& package, grapple_user user)
{
	grapple_server_send(mServer, user, 0, package.data(), package.size());
}

void Server::callback(grapple_message* message)
{
	mMessages.push(message);
}

#define PUSH_MASSAGE(x) case x: mDispatcher->log(#x);
#define PUSH_MASSAGE_SESSION(x) case x: mDispatcher->log(String(#x ": ") + message->SESSION_NAME.name); break;

void Server::processMessage(grapple_message* message)
{
	switch (message->type)
	{
	case GRAPPLE_MSG_USER_MSG:
	{
		ByteArray data(message->USER_MSG.length, reinterpret_cast<unsigned char*>(message->USER_MSG.data));
		Package package(message->USER_MSG.id, data);
		mDispatcher->package(package);
		break;
	}
	case GRAPPLE_MSG_NEW_USER:
	case GRAPPLE_MSG_NEW_USER_ME:
		mDispatcher->newUser(message->NEW_USER.id, message->NEW_USER.me, message->NEW_USER.name);
		break;
	case GRAPPLE_MSG_USER_DISCONNECTED:
		mDispatcher->userDisconnected(message->USER_DISCONNECTED.id);
		break;

		PUSH_MASSAGE(GRAPPLE_MSG_USER_NAME);
		break;
		PUSH_MASSAGE_SESSION(GRAPPLE_MSG_SESSION_NAME);
		break;
		PUSH_MASSAGE(GRAPPLE_MSG_SERVER_DISCONNECTED);
		break;
		PUSH_MASSAGE(GRAPPLE_MSG_CONNECTION_REFUSED);
		break;
		PUSH_MASSAGE(GRAPPLE_MSG_PING);
		break;
		PUSH_MASSAGE(GRAPPLE_MSG_GROUP_CREATE);
		break;
		PUSH_MASSAGE(GRAPPLE_MSG_GROUP_ADD);
		break;
		PUSH_MASSAGE(GRAPPLE_MSG_GROUP_REMOVE);
		break;
		PUSH_MASSAGE(GRAPPLE_MSG_GROUP_DELETE);
		break;
		PUSH_MASSAGE(GRAPPLE_MSG_YOU_ARE_HOST);
		break;
		PUSH_MASSAGE(GRAPPLE_MSG_CONFIRM_RECEIVED);
		break;
		PUSH_MASSAGE(GRAPPLE_MSG_CONFIRM_TIMEOUT);
		break;
		PUSH_MASSAGE(GRAPPLE_MSG_GAME_DESCRIPTION);
		break;
	}

	grapple_message_dispose(message);
}
