/*
 * CG Madness - a Marble Madness clone
 * Copyright (C) 2007  Sven Reinck <sreinck@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "KeyPresser.hpp"

#include <cstdio>
#include <cstring>

KeyPresser::KeyPresser(const char* chars) :
	mTime(0.0f),
	mKeyPressed(false)
{
	char* tmp = new char[strlen(chars) + 1];
	char* p = tmp;

	for (const char* ch = chars; *ch; ch++)
	{
		if (*ch == '\\')
		{
			int oct = 0;
			for (int i = 0; i < 3 && *(++ch); i++)
			{
				oct = 8 * oct + (*ch - '0');
			}

			*(p++) = oct;
		}
		else
		{
			*(p++) = *ch;
		}
	}

	*p = 0;

	mChars = tmp;
	mNextChar = mChars;

	delete[] tmp;
}

KeyPresser::~KeyPresser()
{
  // empty
}

void KeyPresser::start()
{
	takeKeyboard(mKeyDown, mKeyUp);
	mWaitTime = 1.0f;
}

void KeyPresser::stop()
{
	startKeyboard();
	mKeyDown = NULL;
	mKeyUp = NULL;
}

void KeyPresser::update(float interval)
{
	if (mTime < mWaitTime)
	{
		mTime += interval;
	}
	else if (*mNextChar)
	{
		mTime += interval;
		mWaitTime = 0.0f;

		if (mTime < 0.1f)
		{
			if (mKeyPressed)
			{
				mKeyUp(*mNextChar, 0, 0);
				mNextChar++;
				mKeyPressed = false;
			}
		}
		else if (mTime < 0.2f)
		{
			if (!mKeyPressed)
			{
				mKeyDown(*mNextChar, 0, 0);
				printf("KeyDown: %d\n", *mNextChar);
				mKeyPressed = true;
			}
		}
		else
		{
			mTime -= 0.2f;
		}
	}
	else if (mKeyDown)
	{
		stop();
	}
}

void KeyPresser::draw() const
{
  // empty
}
