/*
 * CG Madness - a Marble Madness clone
 * Copyright (C) 2007  Sven Reinck <sreinck@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef SandboxGame_hpp
#define SandboxGame_hpp

#include "process/Game.hpp"
#include "SandboxStub.hpp"
#include "SandboxStatus.hpp"

#include "utils/String.hpp"
#include "utils/Singleton.hpp"

class SandboxGame : public Game, public SandboxStub
{
public:
	typedef enum
	{
		STATE_LOBBY,
		STATE_RUNNING
	} SandboxState;

public:
  SandboxGame();
  virtual ~SandboxGame();

  void start(Process* previous, bool push);
  void stop(Process* next);

  void suspend();
  void resume();

  void setPlayerName(const String& name);
  void chooseLevel();
  void startAllGames();
  void endAllGames();
  void changeMyState(PlayerInfo::PlayerState newState);
  void gotoLobby();
  void leave();
  void disconnect();

  void levelChosen();

  void startServer();
  void startClient(const String& servername);

  void update(float interval);

  void draw() const;
  void drawHUD(float width, float height);

	void drawWaterReflection() const;
	void drawBallReflection() const;

	const String& playerName() const;
	const PlayerMap& playerMap() const;
	const bool& master() const;
	const ConnectionState& connectionState() const;
	const int& reason() const;

public:
  void log(const String& log);

  void newUser(grapple_user id, bool me, const char* name);
  void userName(grapple_user id, const char* name);
  void userDisconnected(grapple_user id);
  void serverDisconnected();
  void connectionRefused(grapple_connection_refused reason);

private:
	SmartPointer<class ServerThread> mServerThread;
	SmartPointer<class Client> mClient;
	SmartPointer<class SandboxProxy> mSandboxProxy;

	SmartPointer<class ScreenSandboxLobby> gScreenSandboxLobby;
	SmartPointer<class ScreenChooseInfo> gScreenChooseInfo;
	SmartPointer<class Choose> mChoose;
	SmartPointer<class ScreenSandboxEnd> gScreenEnd;

	Singleton<class LevelLoader> mLevelLoader;

	String mPlayerName;

	SandboxStatus mStatus;
	Tick mLastUpdate;

	grapple_user mMyID;
	grapple_user mMasterID;
	bool mMaster;
	bool mStatusDirty;

	SandboxState mSandboxState;
	ConnectionState mConnectionState;
	int mReason;

	void finishedGame();

	void activateMaster();

private:
#include "SandboxProxy1.inc"

};

inline const String& SandboxGame::playerName() const
{
	return mPlayerName;
}

inline const PlayerMap& SandboxGame::playerMap() const
{
	return mStatus.playerMap();
}

inline const bool& SandboxGame::master() const
{
	return mMaster;
}

inline const ConnectionState& SandboxGame::connectionState() const
{
	return mConnectionState;
}

inline const int& SandboxGame::reason() const
{
	return mReason;
}

#endif
