/*
 * CG Madness - a Marble Madness clone
 * Copyright (C) 2007  Sven Reinck <sreinck@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "SandboxStatus.hpp"

#include "macros.hpp"

#define TICKS_PER_SECOND 30

SandboxStatus::SandboxStatus()
{
  reset(0);
}

SandboxStatus::~SandboxStatus()
{
  // empty
}

void SandboxStatus::setName(grapple_user id, bool me, const String& name)
{
	mPlayerMap[id].name = name;

	if (!me)
	{
		mOtherBalls[id].mBall.name() = name;
	}
}

void SandboxStatus::setColor(grapple_user id, bool me, const Color3& color)
{
	mPlayerMap[id].color = color;

	if (!me)
	{
		NetworkBall& ball = mOtherBalls[id].mBall;
		ball.color() = color;
	}
}

void SandboxStatus::changeStatusState(grapple_user id, float time, int newState)
{
	mPlayerMap[id].time = time;
	mPlayerMap[id].state = (PlayerInfo::PlayerState) newState;
}

void SandboxStatus::attemptNewTry(grapple_user id, int session, float time)
{
	NetworkBallInfo& info = mOtherBalls[id];

	if (session == mSession && time > info.mLastTime)
	{
		info.mLastTime = time;
		info.mTime = info.mLastTime;

		incTrys(id, time);
	}
}

void SandboxStatus::updatePosition(grapple_user id, int session, float time, const Vector3& pos,
		const Vector3& vel)
{
	NetworkBallInfo& info = mOtherBalls[id];

	if (session == mSession && time > info.mLastTime)
	{
		info.mLastTime = time;
		info.mTime = info.mLastTime;

		NetworkBall& ball = info.mBall;

		ball.pos() = pos;
		ball.velocity() = vel;

		ball.alive();
	}
}

void SandboxStatus::causeExplosion(grapple_user id)
{
	NetworkBall& ball = mOtherBalls[id].mBall;
	ball.explodeBall();
}

void SandboxStatus::erase(grapple_user id)
{
	mPlayerMap.erase(id);
	mOtherBalls.erase(id);
}

void SandboxStatus::clear()
{
	mPlayerMap.clear();
	mOtherBalls.clear();
}

void SandboxStatus::reset(int session)
{
	mSession = session;
	mCurTime = 0.0f;
	mCurTicks = 0;
	mLastTime = mCurTime;
	FOREACH(mOtherBalls, iter)
	{
		iter->second.mBall.reset();
		iter->second.mLastTime = 0.0f;
	}

	FOREACH(mPlayerMap, iter)
	{
		iter->second.trys = 0;
	}
}

int SandboxStatus::nextSession()
{
	mSession++;
	return mSession;
}

void SandboxStatus::incTrys(grapple_user id, float time)
{
	PlayerInfo& info = mPlayerMap[id];

	if (info.trys == 0)
	{
		info.firstStartTime = time;
	}
	info.lastStartTime = time;

	info.trys++;
}

void SandboxStatus::update(float interval)
{
	mCurTime += interval;

	FOREACH(mPlayerMap, iter)
	{
		grapple_user id = iter->first;
		PlayerInfo& info = iter->second;

		if (info.state == PlayerInfo::STATE_PLAYING)
		{
			BallMap::iterator iBall = mOtherBalls.find(id);
			if (iBall != mOtherBalls.end())
			{
				NetworkBallInfo& ballinfo = iBall->second;
				NetworkBall& ball = ballinfo.mBall;
				while (!ball.hasHitGoal() && ballinfo.mTime < mCurTime)
				{
 					ball.update(interval);
					ballinfo.mTime += interval;
				}

				info.time = ballinfo.mTime;
			}
			else
			{
				info.time = mCurTime;
			}
		}
	}

	if ((mCurTime - mLastTime) * TICKS_PER_SECOND >= 1.0f)
	{
		mCurTicks++;
		mLastTime = mCurTime;
	}
}
