/*
 * CG Madness - a Marble Madness clone
 * Copyright (C) 2007  Sven Reinck <sreinck@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef SandboxStatus_hpp
#define SandboxStatus_hpp

#include "ball/NetworkBall.hpp"
#include "PlayerInfo.hpp"

#include "libgrapple/grapple_types.h"

#include <map>

typedef int Tick;

struct NetworkBallInfo
{
	NetworkBall mBall;
	float mTime;
	float mLastTime;
};

class SandboxStatus
{
public:
	typedef std::map<grapple_user, NetworkBallInfo> BallMap;

public:
  SandboxStatus();
  virtual ~SandboxStatus();

  void setName(grapple_user id, bool me, const String& name);
  void setColor(grapple_user id, bool me, const Color3& color);
  void changeStatusState(grapple_user id, float time, int newState);
  void attemptNewTry(grapple_user id, int session, float time);
  void updatePosition(grapple_user id, int session, float time, const Vector3& pos, const Vector3& vel);
  void causeExplosion(grapple_user id);
  void erase(grapple_user id);

  void clear();
  void reset(int session);
  int nextSession();
  void incTrys(grapple_user id, float time);
  void update(float interval);

  const int& session() const;
  const float& time() const;
  const Tick& ticks() const;
  const PlayerMap& playerMap() const;
  const BallMap& ballMap() const;

private:
	int mSession;
	float mCurTime;
	float mLastTime;
	Tick mCurTicks;

	PlayerMap mPlayerMap;
	BallMap mOtherBalls;

};

inline const int& SandboxStatus::session() const
{
	return mSession;
}

inline const float& SandboxStatus::time() const
{
	return mCurTime;
}

inline const Tick& SandboxStatus::ticks() const
{
	return mCurTicks;
}

inline const PlayerMap& SandboxStatus::playerMap() const
{
	return mPlayerMap;
}

inline const SandboxStatus::BallMap& SandboxStatus::ballMap() const
{
	return mOtherBalls;
}

#endif
