/*
 * CG Madness - a Marble Madness clone
 * Copyright (C) 2007  Sven Reinck <sreinck@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "PlayerList.hpp"

#include "graphics/objects.hpp"
#include "text/text.hpp"

#include "macros.hpp"

#include GL_H

#define PADDING_FACTOR 1.8f

PlayerList::PlayerList(unsigned int maxLines, const ConnectionState& state,
    const int& reason, const PlayerMap& playerMap) :
	MenuItem(false),
	mMaxLines(maxLines),
	mState(state),
	mReason(reason),
	mPlayerMap(playerMap)
{
	// empty
}

PlayerList::~PlayerList()
{
	// empty
}

void PlayerList::draw() const
{
	float lineHight = mHeight / mMaxLines;
	float scale = lineHight / PADDING_FACTOR;
	float padding = (lineHight - scale) / 2.0f;

	drawPanel(mWidth, mHeight);

	if (mState != STATE_CONNECTED)
	{
		float y = padding + 0.5f * mMaxLines * lineHight;

		glColor3f(1.0f, 1.0f, 1.0f);

		glPushMatrix();
		{
			const char* strState = sgConnectionStateString[mState];

			glTranslatef(0.5f * (mWidth - widthStrokeText(strState) * scale), y, 0.0f);
			glScalef(scale, scale, scale);

			drawStrokeThinText(strState);
		}
		glPopMatrix();

		y -= lineHight;

		if (mReason != 0)
		{
			glPushMatrix();
			{
				const char* strReason = sgConnectionRefusedString[mReason - 1];

				glTranslatef(0.5f * (mWidth - widthStrokeText(strReason) * scale), y, 0.0f);
				glScalef(scale, scale, scale);

				drawStrokeThinText(strReason);
			}
			glPopMatrix();
		}

		return;
	}

	int idxLine = mMaxLines - 1;

	FOREACH(mPlayerMap, iter)
	{
		const PlayerInfo& info = (*iter).second;
		const char* strState = sgPlayerStateString[info.state];

		float y = padding + idxLine * lineHight;

		glColor3fv(&info.color.r);

		glPushMatrix();
		{
			glTranslatef(padding, y, 0.0f);
			glScalef(scale, scale, scale);

			drawStrokeThinText(info.name);
		}
		glPopMatrix();

		glPushMatrix();
		{
			glTranslatef(mWidth - padding - widthStrokeText(strState) * scale, y, 0.0f);
			glScalef(scale, scale, scale);

			drawStrokeThinText(strState);
		}
		glPopMatrix();

		idxLine--;
	}
}
