/*
 * CG Madness - a Marble Madness clone
 * Copyright (C) 2007  Sven Reinck <sreinck@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "RankingList.hpp"

#include "graphics/objects.hpp"
#include "text/text.hpp"

#include "macros.hpp"

#include GL_H

#include <cstdio>

#define PADDING_FACTOR 1.8f

#define COLUMNS 5

RankingList::RankingList(unsigned int maxLines, const PlayerMap& playerMap) :
	MenuItem(false),
	mMaxLines(maxLines + 1),
	mPlayerMap(playerMap)
{
	// empty
}

RankingList::~RankingList()
{
	// empty
}

String toTimeString(float time)
{
	char strTime[10];
	int tenthSecond = (int) (time * 10);

	sprintf(strTime, "%d:%02d.%01d", tenthSecond / 600, tenthSecond / 10 % 60,			tenthSecond % 10);

	return String(strTime);
}

void RankingList::setSize(float width, float height)
{
	mWidth = width;
	mHeight = height;

	mLineHight = mHeight / mMaxLines;
	mScale = mLineHight / PADDING_FACTOR;
	mPadding = (mLineHight - mScale) / 2.0f;
	mSidePadding = mLineHight;
	mColumnWidth = (mWidth - 2.0f * mSidePadding) / COLUMNS;
}

void RankingList::draw() const
{
	drawPanel(mWidth, mHeight);

	{
		glColor3fv(Color4::white);

		drawTextToCell(0, 0, "name", 0);
		drawTextToCell(1, 0, "finished?", 0);
		drawTextToCell(2, 0, "last pass", 0);
		drawTextToCell(3, 0, "trys", 0);
		drawTextToCell(4, 0, "time", 0);
	}

	int row = 1;

	FOREACH(mPlayerMap, iter)
	{
		const PlayerInfo& info = iter->second;

		glColor3fv(info.color);

		drawTextToCell(0, row, info.name, 1);
		drawTextToCell(1, row, info.state == PlayerInfo::STATE_FINISHED ? "yes" : "no", 0);
		drawTextToCell(2, row, toTimeString(info.time - info.lastStartTime), 0);
		drawTextToCell(3, row, String::toString(info.trys), 0);
		drawTextToCell(4, row, toTimeString(info.time - info.firstStartTime), 0);

		row++;
	}
}

void RankingList::drawTextToCell(int col, int row, const char* text, int align) const
{
	glPushMatrix();
	{
		float x;
		float y = mPadding + (mMaxLines - 1 - row) * mLineHight;

		if (align == 0)
		{
			x = 0.5f * (mColumnWidth - widthStrokeText(text) * mScale);
		}
		else if (align < 0)
		{
			x = mColumnWidth - mPadding - widthStrokeText(text) * mScale;
		}
		else
		{
			x = mPadding;
		}

		x += mSidePadding + mColumnWidth * col;

		glTranslatef(x, y, 0.0f);
		glScalef(mScale, mScale, mScale);

		drawStrokeThinText(text);
	}
	glPopMatrix();
}
