/*
 * CG Madness - a Marble Madness clone
 * Copyright (C) 2007  Sven Reinck <sreinck@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "ScreenSandboxLobby.hpp"

#include "utils/Callback.hpp"

#include "hw/keyboard.hpp"

#define PLAYER_LIST_LINES 10

#define CONSOLE_WIDTH 8.0f
#define CONSOLE_HEIGHT 3.0f

ScreenSandboxLobby::ScreenSandboxLobby(SandboxGame* parent) :
	mParent(parent),
	mPlayerList(PLAYER_LIST_LINES, mParent->connectionState(), mParent->reason(), mParent->playerMap())
{
	mPlayerList.position = Vector2(-CONSOLE_WIDTH / 2.0f, 4.0f);
	mPlayerList.mWidth = CONSOLE_WIDTH;
	mPlayerList.mHeight = CONSOLE_HEIGHT;
	addItem(&mPlayerList);

	bStart = ConditionalButton(3.0f, METHOD_CALLBACK(ScreenSandboxLobby, clickButtonStart), "start", KEY_ENTER,
			&mParent->master());
	addItem(&bStart);

	bChange = ConditionalButton(2.0f, METHOD_CALLBACK(ScreenSandboxLobby, clickButtonChange), "change level", 'c',
			&mParent->master());
	addItem(&bChange);

	bDisconnect = Button(1.0f, METHOD_CALLBACK(ScreenSandboxLobby, clickButtonDisconnect), "disconnect", KEY_ESC);
	addItem(&bDisconnect);
}

ScreenSandboxLobby::~ScreenSandboxLobby()
{
  // empty
}

void ScreenSandboxLobby::clickButtonStart()
{
	mParent->startAllGames();
}

void ScreenSandboxLobby::clickButtonChange()
{
	mParent->chooseLevel();
}

void ScreenSandboxLobby::clickButtonDisconnect()
{
	mParent->disconnect();
}
