/*
 * CG Madness - a Marble Madness clone
 * Copyright (C) 2007  Sven Reinck <sreinck@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "ServerThread.hpp"

#include "macros.hpp"

#include <cstdio>

ServerThread::ServerThread() :
	mMasterUser(GRAPPLE_SERVER)
{
  // empty
}

ServerThread::~ServerThread()
{
  // empty
}

void ServerThread::run()
{
	mServer = new Server(PROGRAM_NAME, MACRO_STRING(PROTOCOL_VERSION), this, &mRunning);

	mSandboxProxy = new SandboxProxy(*mServer);

	mServer->loopMessages();
}

void ServerThread::startServer()
{
	start();
	mRunning.waitFor(true);
}

void ServerThread::stopServer()
{
	mSandboxProxy = NULL;
	mServer = NULL;
	mRunning.waitFor(false);
}

void ServerThread::log(const String& log)
{
	printf("Server: %s\n", (const char*) log);
}

void ServerThread::newUser(grapple_user id, bool me, const char* name)
{
	if (mMasterUser == GRAPPLE_SERVER)
	{
		mMasterUser = id;
	}

	mSandboxProxy->setMasterUser(id, mMasterUser);

	mUsers.insert(id);
}

void ServerThread::userDisconnected(grapple_user id)
{
	mUsers.erase(id);

	if (id == mMasterUser)
	{
		if (mUsers.empty())
		{
			mMasterUser = GRAPPLE_SERVER;
		}
		else
		{
			mMasterUser = *mUsers.begin();
		}
		mSandboxProxy->setMasterUser(GRAPPLE_EVERYONE, mMasterUser);
	}
}

void ServerThread::package(const Package& package)
{
	// empty
}
