/*
 * CG Madness - a Marble Madness clone
 * Copyright (C) 2007  Sven Reinck <sreinck@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef ServerThread_hpp
#define ServerThread_hpp

#include "thread/Thread.hpp"
#include "net/MessageDispatcher.hpp"

#include "sandbox/SandboxProxy.hpp"

#include "net/Server.hpp"

#include "thread/BooleanCondition.hpp"

#include "utils/SmartPointer.hpp"

#include "libgrapple/grapple_types.h"

#include <set>

#define PROGRAM_NAME "CGMadness"
#define PROTOCOL_VERSION 1

class ServerThread : public Thread, public MessageDispatcher
{
public:
  ServerThread();
  virtual ~ServerThread();

  void run();

  void startServer();
  void stopServer();

public:
  void log(const String& log);

  void newUser(grapple_user id, bool me, const char* name);
  void userDisconnected(grapple_user id);

  void package(const Package& package);

private:
	BooleanCondition mRunning;

	SmartPointer<Server> mServer;
	SmartPointer<SandboxProxy> mSandboxProxy;

	grapple_user mMasterUser;
	std::set<grapple_user> mUsers;

};

#endif
