/*
 * CG Madness - a Marble Madness clone
 * Copyright (C) 2007  Sven Reinck <sreinck@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "BooleanCondition.hpp"

#include "Synchronized.hpp"

BooleanCondition::BooleanCondition(bool value) :
	Condition(mLock),
	mValue(value)
{
  // empty
}

BooleanCondition::~BooleanCondition()
{
  // empty
}

void BooleanCondition::waitFor(bool value)
{
	Synchronized sync(mLock);
	
	while (mValue != value)
	{
		wait();
	}
}

void BooleanCondition::setValue(bool value)
{
	Synchronized sync(mLock);
	
	mValue = value;
	
	signal();
}

BooleanCondition::operator bool ()
{
	Synchronized sync(mLock);
	
	return mValue;
}
