/*
 * CG Madness - a Marble Madness clone
 * Copyright (C) 2007  Sven Reinck <sreinck@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "ByteArray.hpp"

#include <string.h>

struct ByteArrayData
{
	unsigned int mCounter;
	unsigned int mSize;
	unsigned char* mData;
	
	ByteArrayData(unsigned int size) :
		mCounter(1),
		mSize(size),
		mData(new unsigned char[mSize])
	{
		// empty
	}
	
	~ByteArrayData()
	{
		delete[] mData;
	}
};

ByteArray::ByteArray(unsigned int size)
{
	mData = new ByteArrayData(size);
}

ByteArray::ByteArray(unsigned int size, const unsigned char* data)
{
	mData = new ByteArrayData(size);
	memcpy(mData->mData, data, mData->mSize);
}

ByteArray::ByteArray(const ByteArray& other)
{
	assign(other);
}

ByteArray::~ByteArray()
{
	release();
}

const unsigned int ByteArray::size() const
{
	return mData->mSize;
}

unsigned char* ByteArray::data()
{
	if (mData->mCounter > 1)
	{
		release();
		
		ByteArrayData* oldData = mData; 
		
		mData = new ByteArrayData(mData->mSize);
		memcpy(mData->mData, oldData->mData, mData->mSize);
	}
	
	return mData->mData;
}

const unsigned char* ByteArray::data() const
{
	return mData->mData;
}

void ByteArray::operator=(const ByteArray& other)
{
	if (mData != other.mData)
	{
		release();
		assign(other);
	}
}
void ByteArray::release()
{
	mData->mCounter--;
	if (mData->mCounter == 0)
	{
		delete mData;
	}
}

void ByteArray::assign(const ByteArray& other)
{
	mData = other.mData;
	mData->mCounter++;
}
